@@ -50,8 +50,12 @@ function inflateShrinkwrap (topPath, tree, swdeps, opts) {
 
   return BB.each(Object.keys(swdeps), (name) => {
     const sw = swdeps[name]
-    const dependencies = sw.dependencies || {}
     const requested = realizeShrinkwrapSpecifier(name, sw, topPath)
+    // We should not muck about in the node_modules folder of
+    // symlinked packages.  Treat its dependencies as if they
+    // were empty, since it's none of our business.
+    const dependencies = requested.type === 'directory' ? {}
+      : sw.dependencies || {}
     return inflatableChild(
       onDisk[name], name, topPath, tree, sw, requested, opts
     ).then((child) => {
@@ -69,6 +73,26 @@ function normalizePackageDataNoErrors (pkg) {
   }
 }
 
+function quotemeta (str) {
+  return str.replace(/([^A-Za-z_0-9/])/g, '\\$1')
+}
+
+function tarballToVersion (name, tb) {
+  const registry = quotemeta(npm.config.get('registry') || '')
+    .replace(/https?:/, 'https?:')
+    .replace(/([^/])$/, '$1/')
+  let matchRegTarball
+  if (name) {
+    const nameMatch = quotemeta(name)
+    matchRegTarball = new RegExp(`^${registry}${nameMatch}/-/${nameMatch}-(.*)[.]tgz$`)
+  } else {
+    matchRegTarball = new RegExp(`^${registry}(.*)?/-/\\1-(.*)[.]tgz$`)
+  }
+  const match = tb.match(matchRegTarball)
+  if (!match) return
+  return match[2] || match[1]
+}
+
 function inflatableChild (onDiskChild, name, topPath, tree, sw, requested, opts) {
   validate('OSSOOOO|ZSSOOOO', arguments)
   const usesIntegrity = (
@@ -76,6 +100,12 @@ function inflatableChild (onDiskChild, name, topPath, tree, sw, requested, opts)
     requested.type === 'remote' ||
     requested.type === 'file'
   )
+  const regTarball = tarballToVersion(name, sw.version)
+  if (regTarball) {
+    sw.resolved = sw.version
+    sw.version = regTarball
+  }
+  if (sw.requires) Object.keys(sw.requires).map(_ => { sw.requires[_] = tarballToVersion(_, sw.requires[_]) || sw.requires[_] })
   const modernLink = requested.type === 'directory' && !sw.from
   if (hasModernMeta(onDiskChild) && childIsEquivalent(sw, requested, onDiskChild)) {
     // The version on disk matches the shrinkwrap entry.
@@ -115,6 +145,10 @@ function isGit (sw) {
 }
 
 function makeFakeChild (name, topPath, tree, sw, requested) {
+  // We should not muck about in the node_modules folder of
+  // symlinked packages.  Treat its dependencies as if they
+  // were empty, since it's none of our business.
+  const isDirectory = requested.type === 'directory'
   const from = sw.from || requested.raw
   const pkg = {
     name: name,
@@ -130,7 +164,7 @@ function makeFakeChild (name, topPath, tree, sw, requested) {
     _spec: requested.rawSpec,
     _where: topPath,
     _args: [[requested.toString(), topPath]],
-    dependencies: sw.requires
+    dependencies: isDirectory ? {} : sw.requires
   }
 
   if (!sw.bundled) {
@@ -141,17 +175,17 @@ function makeFakeChild (name, topPath, tree, sw, requested) {
   }
   const child = createChild({
     package: pkg,
-    loaded: true,
+    loaded: isDirectory,
     parent: tree,
     children: [],
     fromShrinkwrap: requested,
     fakeChild: sw,
     fromBundle: sw.bundled ? tree.fromBundle || tree : null,
     path: childPath(tree.path, pkg),
-    realpath: requested.type === 'directory' ? requested.fetchSpec : childPath(tree.realpath, pkg),
+    realpath: isDirectory ? requested.fetchSpec : childPath(tree.realpath, pkg),
     location: (tree.location === '/' ? '' : tree.location + '/') + pkg.name,
-    isLink: requested.type === 'directory',
-    isInLink: tree.isLink,
+    isLink: isDirectory,
+    isInLink: tree.isLink || tree.isInLink,
     swRequires: sw.requires
   })
   tree.children.push(child)
@@ -169,7 +203,7 @@ function fetchChild (topPath, tree, sw, requested) {
     var isLink = pkg._requested.type === 'directory'
     const child = createChild({
       package: pkg,
-      loaded: false,
+      loaded: isLink,
       parent: tree,
       fromShrinkwrap: requested,
       path: childPath(tree.path, pkg),
